#
# Copyright (c) 2016 Sam4Mobile, 2017 Make.org
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

# We need an initiator node before attempting any config of systemd
run_state = node.run_state[cookbook_name]
return if run_state.nil? || run_state['hosts'].nil?

# Construct unit content from command line options
def options_to_unit(options) # rubocop:disable Metrics/AbcSize
  options_str = options.map do |key, opts|
    [opts].flatten.map do |opt|
      "--#{key.to_s.tr('_', '-')}=#{opt}" unless opt == 'nil'
    end
  end.flatten.join(" \\\n  ")

  unit = node[cookbook_name]['unit'].to_hash
  bin = "#{node[cookbook_name]['prefix_home']}/cockroachdb/cockroach"
  unit['Service']['ExecStart'] = "#{bin} start #{options_str}"
  unit
end

# Cli opts for unit, map based
options = node.run_state[cookbook_name]['options']

# Test if there is an existing cluster
# If not, create one (initiator only)
if node.run_state[cookbook_name]['iam_initiator']
  up = options['join'].split(',').map { |h| h.split(':').first }.map do |host|
    out = shell_out("curl -k -L #{host}:8080/health").stdout
    out.include?('nodeId')
  end

  unless up.any?
    # no cluster, let's create one
    # No join option for init
    opts_wo_join = options.dup
    opts_wo_join.delete('join')

    unit = options_to_unit(opts_wo_join)
    systemd_unit 'init cockroachdb.service' do
      name 'cockroachdb.service'
      content unit
      action %i[create start]
      notifies :restart, 'systemd_unit[cockroachdb.service]', :delayed
    end
  end
end

unit = options_to_unit(options)
systemd_unit 'cockroachdb.service' do
  content unit
  action %i[create enable start]
end
